%% This file is used to plot the relation between steady-state equilibrium and capital control
clear
testvar = 1; 
% testvar = 1 to plot steady state effect of TAUD (steady_state_TAUD.pdf)
% testvar = 2 to plot steady state effect of TAUL (steady_state_TAUL.pdf) 

%% calibration
T = 10;
DELTA = 1-0.9^T;
ETA = 2;
BETA = 0.96^T;% imply annualized domestic interest rate = 0.04
ALPHA = 0.5;
THETAS = 1;
THETAP = 1;
AS  = 1;
AP = 1.42;
RSTAR = 1.05^T;
PHI = 0.5;
SIGMA  = 3;
OMEGAK = 5;
PSIH = 24;
THETAY = 1;
PHI = 0.5;
GAMMA = 0.5;
TAU = 0.75;% target annual household consumption to net worth ratio = (cy + co)/10/(d+bfd+k) = 0.07 based on 2011 CHFS survey
PHIB  = 3;
BYBAR = 0.04;

BFL_Y0 = 0.04;
BFD_Y0 = 0.06;
x  = fsolve(@(x) solver(BFL_Y0,BFD_Y0,DELTA,BETA,ALPHA,THETAS,THETAP,AS,AP,RSTAR,PHI,GAMMA,x(1),x(2),TAU,SIGMA,THETAY,PHIB,BYBAR),[0.1 0.1]);
TAUL = x(1);
TAUD = x(2);

%% steady state analysis for capital outflow tax
if testvar == 1
 vTAUD = 0:0.01:0.8;
 for j = 1: 1: length(vTAUD)
 TAUD = vTAUD(j);

[cy,co,h,k,ko,i,d,bs,bp,bfl,bfd,y,ys,yp,hs,hp,ks,kp,w,rk,qk,r,rs,rp,rf,ps,pp,wealth,bg]= solvess_close(DELTA,PSIH,ETA,BETA,ALPHA,THETAS,THETAP,AS,AP,RSTAR,PHI,GAMMA,TAUL,TAUD,TAU,SIGMA,THETAY,PHIB,BYBAR);
if (1-TAUD)*rf>r | rp*(1-TAUL) > rf*exp(PHIB*(bfl/y-BYBAR))
       [x,Fval,flag] = fsolve(@(x) solveshare_inflow(x(1),x(2),x(3),DELTA,BETA,ALPHA,THETAS,THETAP,AS,AP,RSTAR,PHI,GAMMA,TAUL,TAUD,TAU,SIGMA,THETAY,PHIB,BYBAR),[1,0.5,1],optimset('TolFun',1e-12));
       [cy,co,h,k,ko,i,d,bs,bp,bfl,bfd,y,ys,yp,hs,hp,ks,kp,w,rk,qk,r,rs,rp,rf,ps,pp,wealth,bg]= solvess_inflow(DELTA,PSIH,ETA,BETA,ALPHA,THETAS,THETAP,AS,AP,RSTAR,PHI,GAMMA,TAUL,TAUD,TAU,SIGMA,THETAY,PHIB,BYBAR);
    if (1-TAUD)*rf>r | bfl<0 | bfd<0 | flag<1 | abs(imag(h))>0
       [x,Fval,flag] = fsolve(@(x) solveshare_outflow(x(1),DELTA,BETA,ALPHA,THETAS,THETAP,AS,AP,RSTAR,PHI,GAMMA,TAUL,TAUD,TAU,SIGMA,THETAY,PHIB,BYBAR),1,optimset('TolFun',1e-12));
       [cy,co,h,k,ko,i,d,bs,bp,bfl,bfd,y,ys,yp,hs,hp,ks,kp,w,rk,qk,r,rs,rp,rf,ps,pp,wealth,bg]= solvess_outflow(DELTA,PSIH,ETA,BETA,ALPHA,THETAS,THETAP,AS,AP,RSTAR,PHI,GAMMA,TAUL,TAUD,TAU,SIGMA,THETAY,PHIB,BYBAR);
       if rp*(1-TAUL) > rf*exp(PHIB*(bfl/y-BYBAR)) | bfd<0 |flag<1| abs(imag(h))>0
    [x,Fval,flag] = fsolve(@(x) solveshare(x(1),x(2),DELTA,BETA,ALPHA,THETAS,THETAP,AS,AP,RSTAR,PHI,GAMMA,TAUL,TAUD,TAU,SIGMA,THETAY,PHIB,BYBAR),[1,0.5],optimset('TolFun',1e-12));
    [cy,co,h,k,ko,i,d,bs,bp,bfl,bfd,y,ys,yp,hs,hp,ks,kp,w,rk,qk,r,rs,rp,rf,ps,pp,wealth,bg]= solvess(DELTA,PSIH,ETA,BETA,ALPHA,THETAS,THETAP,AS,AP,RSTAR,PHI,GAMMA,TAUL,TAUD,TAU,SIGMA,THETAY,PHIB,BYBAR);

       end 
   end  
end


vrp(j) = rp;
vrs(j) = rs;
vutility(j) = log(cy) - PSIH*h^(ETA+1)/(1+ETA) + log(co);
vd(j) = d;
vys(j) = ys;
vyp(j) = yp;
vbs(j) = bs;
vbp(j) = bp;
vc(j) = cy + co;
vw(j) = w;
vi(j) = i;
vy(j) = y;
vnx(j) = y - cy - co - i;
vks(j) = ks;
vkp(j) = kp;
vhs(j) = hs;
vhp(j) = hp;
vtfp(j) = y/(k)^(1-ALPHA)/(h)^ALPHA;
vbfl(j) = bfl;
vbfd(j) = bfd;
vr(j)  = r;
vcy(j) = cy;
vco(j) = co;
vh(j) = h;
vrk(j) = rk;
vps(j) = ps;
vpp(j) = pp;
vbg(j) = bg;
 end


[M,I] = max(vutility);


figure;
subplot(3,2,1)
plot(vTAUD,vbfd,'LineStyle','-','LineWidth',1.5)
title('Capital outflow')
subplot(3,2,2)
plot(vTAUD,vbfl,'LineStyle','-','LineWidth',1.5)
title('Capital inflow')
subplot(3,2,3)
plot(vTAUD,vr,'LineStyle','-','LineWidth',1.5)
title('Domestic deposit rate')
subplot(3,2,4)
plot(vTAUD,vrp,'LineStyle','-','LineWidth',1.5)
title('Market loan rate')
subplot(3,2,5)
plot(vTAUD,vtfp,'LineStyle','-','LineWidth',1.5)
title('Aggregate TFP')
xlabel('Outflow tax \tau_d')
subplot(3,2,6)
plot(vTAUD,vutility,'LineStyle','-','LineWidth',1.5)
title('Utility')
print -dpdf Figures/steady_state_TAUD.pdf;


return
end   
   
%% steady state analysis for capital inflow tax
if testvar == 2
     vTAUL = 0:0.01:0.8;
 for j = 1: 1: length(vTAUL)
 TAUL = vTAUL(j);

[x,Fval,flag] = fsolve(@(x) solveshare(x(1),x(2),DELTA,BETA,ALPHA,THETAS,THETAP,AS,AP,RSTAR,PHI,GAMMA,TAUL,TAUD,TAU,SIGMA,THETAY,PHIB,BYBAR),[1,0.5],optimset('TolFun',1e-12));
[cy,co,h,k,ko,i,d,bs,bp,bfl,bfd,y,ys,yp,hs,hp,ks,kp,w,rk,qk,r,rs,rp,rf,ps,pp,wealth,bg]= solvess(DELTA,PSIH,ETA,BETA,ALPHA,THETAS,THETAP,AS,AP,RSTAR,PHI,GAMMA,TAUL,TAUD,TAU,SIGMA,THETAY,PHIB,BYBAR);
if bfd< 0 | bfl <0 | flag<1 | abs(imag(h))>0 | isnan(h) == 1
    [x,Fval,flag] = fsolve(@(x) solveshare_inflow(x(1),x(2),x(3),DELTA,BETA,ALPHA,THETAS,THETAP,AS,AP,RSTAR,PHI,GAMMA,TAUL,TAUD,TAU,SIGMA,THETAY,PHIB,BYBAR),[1,0.5,1],optimset('TolFun',1e-12));
    [cy,co,h,k,ko,i,d,bs,bp,bfl,bfd,y,ys,yp,hs,hp,ks,kp,w,rk,qk,r,rs,rp,rf,ps,pp,wealth,bg]= solvess_inflow(DELTA,PSIH,ETA,BETA,ALPHA,THETAS,THETAP,AS,AP,RSTAR,PHI,GAMMA,TAUL,TAUD,TAU,SIGMA,THETAY,PHIB,BYBAR);
   if bfl<0 |flag<1 | abs(imag(h))>0 |isnan(h) == 1
       [x,Fval,flag] = fsolve(@(x) solveshare_outflow(x(1),DELTA,BETA,ALPHA,THETAS,THETAP,AS,AP,RSTAR,PHI,GAMMA,TAUL,TAUD,TAU,SIGMA,THETAY,PHIB,BYBAR),1,optimset('TolFun',1e-12));
       [cy,co,h,k,ko,i,d,bs,bp,bfl,bfd,y,ys,yp,hs,hp,ks,kp,w,rk,qk,r,rs,rp,rf,ps,pp,wealth,bg]= solvess_outflow(DELTA,PSIH,ETA,BETA,ALPHA,THETAS,THETAP,AS,AP,RSTAR,PHI,GAMMA,TAUL,TAUD,TAU,SIGMA,THETAY,PHIB,BYBAR);
       if bfd<0 |flag<1| abs(imag(h))>0 |isnan(h) == 1
       [cy,co,h,k,ko,i,d,bs,bp,bfl,bfd,y,ys,yp,hs,hp,ks,kp,w,rk,qk,r,rs,rp,rf,ps,pp,wealth,bg]= solvess_close(DELTA,PSIH,ETA,BETA,ALPHA,THETAS,THETAP,AS,AP,RSTAR,PHI,GAMMA,TAUL,TAUD,TAU,SIGMA,THETAY,PHIB,BYBAR);
       end 
   end  
end

vrp(j) = rp;
vrs(j) = rs;
vutility(j) = log(cy) - PSIH*h^(ETA+1)/(1+ETA) +  log(co);
vd(j) = d;
vys(j) = ys;
vyp(j) = yp;
vbs(j) = bs;
vbp(j) = bp;
vc(j) = cy + co;
vw(j) = w;
vi(j) = i;
vy(j) = y;
vnx(j) = y - cy - co - i;
vks(j) = ks;
vkp(j) = kp;
vhs(j) = hs;
vhp(j) = hp;
vtfp(j) = y/(k)^(1-ALPHA)/(h)^ALPHA;
vbfl(j) = bfl;
vbfd(j) = bfd;
vr(j)  = r;
vcy(j) = cy;
vco(j) = co;
vh(j) = h;
vrk(j) = rk;
vps(j) = ps;
vpp(j) = pp;
vbg(j) = bg;
vwealth(j) = wealth;
 end


[M,I] = max(vutility);


figure;
subplot(3,2,1)
plot(vTAUL,vbfd,'LineStyle','-','LineWidth',1.5)
title('Capital outflow')
subplot(3,2,2)
plot(vTAUL,vbfl,'LineStyle','-','LineWidth',1.5)
title('Capital inflow')
subplot(3,2,3)
plot(vTAUL,vr,'LineStyle','-','LineWidth',1.5)
title('Domestic deposit rate')
subplot(3,2,4)
plot(vTAUL,vrp,'LineStyle','-','LineWidth',1.5)
title('Market loan rate')
subplot(3,2,5)
plot(vTAUL,vtfp,'LineStyle','-','LineWidth',1.5)
title('Aggregate TFP')
xlabel('Inflow tax \tau_l')
subplot(3,2,6)
plot(vTAUL,vutility,'LineStyle','-','LineWidth',1.5)
title('Utility')

print -dpdf Figures/steady_state_TAUL.pdf;

return
end


