%% This file is used to plot the optimal capital control policies under different degrees of financial repression (gamma)
clear
opt_policy = 0; 
% set opt_policy = 1 if you want to do the optimization. 
% set opt_policy = 0 if you want to just use the saved optimal policy (optim_GAMMA.mat)

%% calibration
T = 10;
DELTA = 1-0.9^T;
ETA = 2;
BETA = 0.96^T;% imply annualized domestic interest rate = 0.04
ALPHA = 0.5;
THETAS = 1;
THETAP = 1;
AS  = 1;
AP = 1.42;
RSTAR = 1.05^T;
PHI = 0.5;
SIGMA  = 3;
OMEGAK = 5;
PSIH = 24;
THETAY = 1;
PHI = 0.5;
GAMMA = 0.5;
TAU = 0.75;% target annual household consumption to net worth ratio = (cy + co)/10/(d+bfd+k) = 0.07 based on 2011 CHFS survey
PHIB  = 3;
BYBAR = 0.04;

BFL_Y0 = 0.04;
BFD_Y0 = 0.06;
x  = fsolve(@(x) solver(BFL_Y0,BFD_Y0,DELTA,BETA,ALPHA,THETAS,THETAP,AS,AP,RSTAR,PHI,GAMMA,x(1),x(2),TAU,SIGMA,THETAY,PHIB,BYBAR),[0.1 0.1]);
TAUL = x(1);
TAUD = x(2);

TAUL0 = TAUL;
TAUD0 = TAUD;

%% solve for optimal steady-state equilibrium under different degrees of financial repression
vGAMMA = [0:0.01:0.7]
for j = 1:1:length(vGAMMA)
GAMMA = vGAMMA(j);

% solve for optimal capital control policies
if opt_policy == 1

[X,FVAL,EXITFLAG]  = fminsearch(@(x) solveU(DELTA,PSIH,ETA,BETA,ALPHA,THETAS,THETAP,AS,AP,RSTAR,PHI,GAMMA,1-exp(x(1)),1-exp(x(2)),TAU,SIGMA,THETAY,PHIB,BYBAR),[0 0]);

vTAUL(j) = 1-exp(X(1));
vTAUD(j) = 1-exp(X(2));
vEXITFLAG(j) = EXITFLAG;
end

if opt_policy == 0
load optim_GAMMA;
end

% solve for steady state equilibrium under optimal capital control policies
TAUL = vTAUL(j);
TAUD = vTAUD(j);

    [cy,co,h,k,ko,i,d,bs,bp,bfl,bfd,y,ys,yp,hs,hp,ks,kp,w,rk,qk,r,rs,rp,rf,ps,pp,wealth,bg]= solvess_close(DELTA,PSIH,ETA,BETA,ALPHA,THETAS,THETAP,AS,AP,RSTAR,PHI,GAMMA,TAUL,TAUD,TAU,SIGMA,THETAY,PHIB,BYBAR);
if (1-TAUD)*rf>r | rp*(1-TAUL) > rf*exp(PHIB*(bfl/y-BYBAR)) 
       [x,Fval,flag] = fsolve(@(x) solveshare_inflow(x(1),x(2),x(3),DELTA,BETA,ALPHA,THETAS,THETAP,AS,AP,RSTAR,PHI,GAMMA,TAUL,TAUD,TAU,SIGMA,THETAY,PHIB,BYBAR),[1,0.5,1],optimset('TolFun',1e-12));
       [cy,co,h,k,ko,i,d,bs,bp,bfl,bfd,y,ys,yp,hs,hp,ks,kp,w,rk,qk,r,rs,rp,rf,ps,pp,wealth,bg]= solvess_inflow(DELTA,PSIH,ETA,BETA,ALPHA,THETAS,THETAP,AS,AP,RSTAR,PHI,GAMMA,TAUL,TAUD,TAU,SIGMA,THETAY,PHIB,BYBAR);
    if (1-TAUD)*rf>r | bfl<0 | bfd<0 | flag<1 | abs(imag(h))>0 | isnan(h) == 1
       [x,Fval,flag] = fsolve(@(x) solveshare_outflow(x(1),DELTA,BETA,ALPHA,THETAS,THETAP,AS,AP,RSTAR,PHI,GAMMA,TAUL,TAUD,TAU,SIGMA,THETAY,PHIB,BYBAR),1,optimset('TolFun',1e-12));
       [cy,co,h,k,ko,i,d,bs,bp,bfl,bfd,y,ys,yp,hs,hp,ks,kp,w,rk,qk,r,rs,rp,rf,ps,pp,wealth,bg]= solvess_outflow(DELTA,PSIH,ETA,BETA,ALPHA,THETAS,THETAP,AS,AP,RSTAR,PHI,GAMMA,TAUL,TAUD,TAU,SIGMA,THETAY,PHIB,BYBAR);
       if rp*(1-TAUL) > rf*exp(PHIB*(bfl/y-BYBAR)) | bfd<0 |flag<1| abs(imag(h))>0 | isnan(h) == 1
    [x,Fval,flag] = fsolve(@(x) solveshare(x(1),x(2),DELTA,BETA,ALPHA,THETAS,THETAP,AS,AP,RSTAR,PHI,GAMMA,TAUL,TAUD,TAU,SIGMA,THETAY,PHIB,BYBAR),[1,0.5],optimset('TolFun',1e-12));
    [cy,co,h,k,ko,i,d,bs,bp,bfl,bfd,y,ys,yp,hs,hp,ks,kp,w,rk,qk,r,rs,rp,rf,ps,pp,wealth,bg]= solvess(DELTA,PSIH,ETA,BETA,ALPHA,THETAS,THETAP,AS,AP,RSTAR,PHI,GAMMA,TAUL,TAUD,TAU,SIGMA,THETAY,PHIB,BYBAR);
    end 
   end  
end

vrp(j) = rp;
vrs(j) = rs;
vutility(j) = log(cy) - PSIH*h^(ETA+1)/(1+ETA) + log(co);
vd(j) = d;
vys(j) = ys;
vyp(j) = yp;
vbs(j) = bs;
vbp(j) = bp;
vc(j) = cy + co;
vw(j) = w;
vi(j) = i;
vy(j) = y;
vnx(j) = y - cy - co - i;
vks(j) = ks;
vkp(j) = kp;
vhs(j) = hs;
vhp(j) = hp;
vtfp(j) = y/(k)^(1-ALPHA)/(h)^ALPHA;
vbfl(j) = bfl;
vbfd(j) = bfd;
vr(j)  = r;
vcy(j) = cy;
vco(j) = co;
vh(j) = h;
vrk(j) = rk;
vps(j) = ps;
vpp(j) = pp;
vbg(j) = bg;

end

%% plot out optimal steady-state equilibrium under different degrees of financial repression

figure;
subplot(3,2,1)
plot(vGAMMA,vTAUD,'LineStyle','-','LineWidth',1.5)
title('Capital outflow tax')
subplot(3,2,2)
plot(vGAMMA,vTAUL,'LineStyle','-','LineWidth',1.5)
title('Capital inflow tax')
subplot(3,2,3)
plot(vGAMMA,vr,'LineStyle','-','LineWidth',1.5)
title('Domestic deposit rate')
subplot(3,2,4)
plot(vGAMMA,vrp,'LineStyle','-','LineWidth',1.5)
title('POE funding cost')
subplot(3,2,5)
plot(vGAMMA,vtfp,'LineStyle','-','LineWidth',1.5)
title('Aggregate TFP')
xlabel('Financial repression \gamma')
subplot(3,2,6)
plot(vGAMMA,vutility,'LineStyle','-','LineWidth',1.5)
title('Utility')

print -dpdf Figures/optim_GAMMA.pdf;

if opt_policy == 1
save optim_GAMMA vTAUD vTAUL vEXITFLAG
end


return


