function [residual, g1, g2, g3] = model_det1_dynamic(y, x, params, steady_state, it_)
%
% Status : Computes dynamic model for Dynare
%
% Inputs :
%   y         [#dynamic variables by 1] double    vector of endogenous variables in the order stored
%                                                 in M_.lead_lag_incidence; see the Manual
%   x         [M_.exo_nbr by nperiods] double     matrix of exogenous variables (in declaration order)
%                                                 for all simulation periods
%   params    [M_.param_nbr by 1] double          vector of parameter values in declaration order
%   it_       scalar double                       time period for exogenous variables for which to evaluate the model
%
% Outputs:
%   residual  [M_.endo_nbr by 1] double    vector of residuals of the dynamic model equations in order of 
%                                          declaration of the equations
%   g1        [M_.endo_nbr by #dynamic variables] double    Jacobian matrix of the dynamic model equations;
%                                                           rows: equations in order of declaration
%                                                           columns: variables in order stored in M_.lead_lag_incidence
%   g2        [M_.endo_nbr by (#dynamic variables)^2] double   Hessian matrix of the dynamic model equations;
%                                                              rows: equations in order of declaration
%                                                              columns: variables in order stored in M_.lead_lag_incidence
%   g3        [M_.endo_nbr by (#dynamic variables)^3] double   Third order derivative matrix of the dynamic model equations;
%                                                              rows: equations in order of declaration
%                                                              columns: variables in order stored in M_.lead_lag_incidence
%
%
% Warning : this file is generated automatically by Dynare
%           from model file (.mod)

%
% Model equations
%

residual = zeros(32, 1);
T43 = y(15)/y(14)-(steady_state(6))/(steady_state(5));
T45 = params(11)/2*T43^2;
T99 = y(10)*params(4)/y(42);
T134 = (params(13)-1)/params(13);
T141 = x(it_, 4)*y(22)^T134+(1-x(it_, 4))*y(23)^T134;
T227 = exp(params(15)*(y(20)/y(21)-params(16)));
T248 = y(1)^(1-params(5));
T249 = y(21)/T248;
T250 = y(12)^params(5);
T302 = params(11)/2*(-y(15))/(y(14)*y(14))*2*T43;
T314 = params(11)/2*2*T43*1/y(14);
T365 = getPowerDeriv(T141,params(13)/(params(13)-1),1);
lhs =y(39);
rhs =log(y(10))-params(2)*y(12)^(1+params(3))/(1+params(3))+log(y(11))+params(4)*y(45);
residual(1)= lhs-rhs;
lhs =y(10)+y(16)+y(19)+y(31)*y(14)+y(15)+y(14)*T45;
rhs =y(12)*y(29)+y(38)*params(12)+params(14)*(x(it_-1, 2)*y(8)*y(6)+x(it_-1, 3)*y(9)*y(5));
residual(2)= lhs-rhs;
lhs =y(11);
rhs =y(38)*(1-params(12));
residual(3)= lhs-rhs;
lhs =y(38);
rhs =y(7)*y(4)+y(5)*y(9)*(1-x(it_-1, 3))+(y(31)*(1-params(1))+y(30))*(y(2)+y(3))+(x(it_-1, 2)*y(8)*y(6)+x(it_-1, 3)*y(9)*y(5))*(1-params(14));
residual(4)= lhs-rhs;
lhs =y(29);
rhs =y(10)*params(2)*y(12)^params(3);
residual(5)= lhs-rhs;
lhs =1;
rhs =T99*y(32);
residual(6)= lhs-rhs;
residual(7) = min(y(32)-(1-x(it_, 3))*y(35),y(19));
lhs =y(31)+T45-y(15)*params(11)*T43/y(14);
rhs =T99*((1-params(1))*y(44)+y(43));
residual(8)= lhs-rhs;
lhs =1+params(11)*T43;
rhs =T99*((1-params(1))*y(44)+y(43));
residual(9)= lhs-rhs;
lhs =y(13);
rhs =y(14)+y(15);
residual(10)= lhs-rhs;
lhs =y(14);
rhs =(1-params(1))*y(1);
residual(11)= lhs-rhs;
lhs =y(21);
rhs =T141^(params(13)/(params(13)-1));
residual(12)= lhs-rhs;
lhs =y(22)*y(36)^params(13);
rhs =y(21)*x(it_, 4)^params(13);
residual(13)= lhs-rhs;
lhs =y(23)*y(37)^params(13);
rhs =y(21)*(1-x(it_, 4))^params(13);
residual(14)= lhs-rhs;
lhs =y(22);
rhs =params(8)*y(26)^(1-params(5))*y(24)^params(5);
residual(15)= lhs-rhs;
lhs =y(17)+y(28);
rhs =params(6)*(y(29)*y(24)+y(30)*y(26));
residual(16)= lhs-rhs;
lhs =y(29)*y(24)*(1-params(6)+params(6)*y(33));
rhs =y(36)*y(22)*params(5);
residual(17)= lhs-rhs;
lhs =y(30)*y(26)*(1-params(6)+params(6)*y(33));
rhs =y(36)*y(22)*(1-params(5));
residual(18)= lhs-rhs;
lhs =y(23);
rhs =params(9)*y(27)^(1-params(5))*y(25)^params(5);
residual(19)= lhs-rhs;
lhs =y(18);
rhs =params(7)*(y(29)*y(25)+y(30)*y(27));
residual(20)= lhs-rhs;
lhs =y(29)*y(25)*(1-params(7)+params(7)*y(34));
rhs =y(37)*y(23)*params(5);
residual(21)= lhs-rhs;
lhs =y(30)*y(27)*(1-params(7)+params(7)*y(34));
rhs =y(37)*y(23)*(1-params(5));
residual(22)= lhs-rhs;
lhs =y(28);
rhs =y(16)*x(it_, 1);
residual(23)= lhs-rhs;
lhs =y(34)*(1-x(it_, 2));
rhs =y(35)*T227;
residual(24)= lhs-rhs;
lhs =y(16)+y(20);
rhs =y(17)+y(28)+y(18);
residual(25)= lhs-rhs;
lhs =y(32);
rhs =x(it_, 1)+y(34)*(1-x(it_, 1));
residual(26)= lhs-rhs;
lhs =y(33);
rhs =(y(28)+y(17)*y(34))/(y(17)+y(28));
residual(27)= lhs-rhs;
lhs =y(12);
rhs =y(24)+y(25);
residual(28)= lhs-rhs;
lhs =y(1);
rhs =y(26)+y(27);
residual(29)= lhs-rhs;
lhs =y(35);
rhs =params(10);
residual(30)= lhs-rhs;
lhs =y(40);
rhs =T249/T250;
residual(31)= lhs-rhs;
lhs =y(41);
rhs =(y(17)+y(28))/y(18);
residual(32)= lhs-rhs;
if nargout >= 2,
  g1 = zeros(32, 49);

  %
  % Jacobian matrix
  %

  g1(1,10)=(-(1/y(10)));
  g1(1,11)=(-(1/y(11)));
  g1(1,12)=params(2)*getPowerDeriv(y(12),1+params(3),1)/(1+params(3));
  g1(1,39)=1;
  g1(1,45)=(-params(4));
  g1(2,10)=1;
  g1(2,12)=(-y(29));
  g1(2,14)=y(31)+T45+y(14)*T302;
  g1(2,15)=1+y(14)*T314;
  g1(2,16)=1;
  g1(2,5)=(-(params(14)*x(it_-1, 3)*y(9)));
  g1(2,19)=1;
  g1(2,6)=(-(params(14)*x(it_-1, 2)*y(8)));
  g1(2,29)=(-y(12));
  g1(2,31)=y(14);
  g1(2,8)=(-(params(14)*x(it_-1, 2)*y(6)));
  g1(2,9)=(-(params(14)*x(it_-1, 3)*y(5)));
  g1(2,38)=(-params(12));
  g1(2,47)=(-(params(14)*y(8)*y(6)));
  g1(2,48)=(-(params(14)*y(9)*y(5)));
  g1(3,11)=1;
  g1(3,38)=(-(1-params(12)));
  g1(4,2)=(-(y(31)*(1-params(1))+y(30)));
  g1(4,3)=(-(y(31)*(1-params(1))+y(30)));
  g1(4,4)=(-y(7));
  g1(4,5)=(-(y(9)*(1-x(it_-1, 3))+x(it_-1, 3)*y(9)*(1-params(14))));
  g1(4,6)=(-(x(it_-1, 2)*y(8)*(1-params(14))));
  g1(4,30)=(-(y(2)+y(3)));
  g1(4,31)=(-((1-params(1))*(y(2)+y(3))));
  g1(4,7)=(-y(4));
  g1(4,8)=(-((1-params(14))*x(it_-1, 2)*y(6)));
  g1(4,9)=(-(y(5)*(1-x(it_-1, 3))+(1-params(14))*x(it_-1, 3)*y(5)));
  g1(4,38)=1;
  g1(4,47)=(-((1-params(14))*y(8)*y(6)));
  g1(4,48)=(-(y(5)*(-y(9))+(1-params(14))*y(9)*y(5)));
  g1(5,10)=(-(params(2)*y(12)^params(3)));
  g1(5,12)=(-(y(10)*params(2)*getPowerDeriv(y(12),params(3),1)));
  g1(5,29)=1;
  g1(6,10)=(-(y(32)*params(4)/y(42)));
  g1(6,42)=(-(y(32)*(-(y(10)*params(4)))/(y(42)*y(42))));
  g1(6,32)=(-T99);
  g1(7,19)=1-(y(19)>y(32)-(1-x(it_, 3))*y(35));
  g1(7,32)=y(19)>y(32)-(1-x(it_, 3))*y(35);
  g1(7,35)=(y(19)>y(32)-(1-x(it_, 3))*y(35))*(-(1-x(it_, 3)));
  g1(7,48)=y(35)*(y(19)>y(32)-(1-x(it_, 3))*y(35));
  g1(8,10)=(-(((1-params(1))*y(44)+y(43))*params(4)/y(42)));
  g1(8,42)=(-(((1-params(1))*y(44)+y(43))*(-(y(10)*params(4)))/(y(42)*y(42))));
  g1(8,14)=T302-(y(14)*y(15)*params(11)*(-y(15))/(y(14)*y(14))-y(15)*params(11)*T43)/(y(14)*y(14));
  g1(8,15)=T314-(params(11)*T43+y(15)*params(11)*1/y(14))/y(14);
  g1(8,43)=(-T99);
  g1(8,31)=1;
  g1(8,44)=(-((1-params(1))*T99));
  g1(9,10)=(-(((1-params(1))*y(44)+y(43))*params(4)/y(42)));
  g1(9,42)=(-(((1-params(1))*y(44)+y(43))*(-(y(10)*params(4)))/(y(42)*y(42))));
  g1(9,14)=params(11)*(-y(15))/(y(14)*y(14));
  g1(9,15)=params(11)*1/y(14);
  g1(9,43)=(-T99);
  g1(9,44)=(-((1-params(1))*T99));
  g1(10,13)=1;
  g1(10,14)=(-1);
  g1(10,15)=(-1);
  g1(11,1)=(-(1-params(1)));
  g1(11,14)=1;
  g1(12,21)=1;
  g1(12,22)=(-(x(it_, 4)*getPowerDeriv(y(22),T134,1)*T365));
  g1(12,23)=(-(T365*(1-x(it_, 4))*getPowerDeriv(y(23),T134,1)));
  g1(12,49)=(-(T365*(y(22)^T134-y(23)^T134)));
  g1(13,21)=(-(x(it_, 4)^params(13)));
  g1(13,22)=y(36)^params(13);
  g1(13,36)=y(22)*getPowerDeriv(y(36),params(13),1);
  g1(13,49)=(-(y(21)*getPowerDeriv(x(it_, 4),params(13),1)));
  g1(14,21)=(-((1-x(it_, 4))^params(13)));
  g1(14,23)=y(37)^params(13);
  g1(14,37)=y(23)*getPowerDeriv(y(37),params(13),1);
  g1(14,49)=(-(y(21)*(-(getPowerDeriv(1-x(it_, 4),params(13),1)))));
  g1(15,22)=1;
  g1(15,24)=(-(params(8)*y(26)^(1-params(5))*getPowerDeriv(y(24),params(5),1)));
  g1(15,26)=(-(y(24)^params(5)*params(8)*getPowerDeriv(y(26),1-params(5),1)));
  g1(16,17)=1;
  g1(16,24)=(-(y(29)*params(6)));
  g1(16,26)=(-(y(30)*params(6)));
  g1(16,28)=1;
  g1(16,29)=(-(y(24)*params(6)));
  g1(16,30)=(-(y(26)*params(6)));
  g1(17,22)=(-(y(36)*params(5)));
  g1(17,24)=y(29)*(1-params(6)+params(6)*y(33));
  g1(17,29)=y(24)*(1-params(6)+params(6)*y(33));
  g1(17,33)=params(6)*y(29)*y(24);
  g1(17,36)=(-(y(22)*params(5)));
  g1(18,22)=(-(y(36)*(1-params(5))));
  g1(18,26)=y(30)*(1-params(6)+params(6)*y(33));
  g1(18,30)=y(26)*(1-params(6)+params(6)*y(33));
  g1(18,33)=params(6)*y(30)*y(26);
  g1(18,36)=(-(y(22)*(1-params(5))));
  g1(19,23)=1;
  g1(19,25)=(-(params(9)*y(27)^(1-params(5))*getPowerDeriv(y(25),params(5),1)));
  g1(19,27)=(-(y(25)^params(5)*params(9)*getPowerDeriv(y(27),1-params(5),1)));
  g1(20,18)=1;
  g1(20,25)=(-(y(29)*params(7)));
  g1(20,27)=(-(y(30)*params(7)));
  g1(20,29)=(-(y(25)*params(7)));
  g1(20,30)=(-(y(27)*params(7)));
  g1(21,23)=(-(y(37)*params(5)));
  g1(21,25)=y(29)*(1-params(7)+params(7)*y(34));
  g1(21,29)=y(25)*(1-params(7)+params(7)*y(34));
  g1(21,34)=params(7)*y(29)*y(25);
  g1(21,37)=(-(y(23)*params(5)));
  g1(22,23)=(-(y(37)*(1-params(5))));
  g1(22,27)=y(30)*(1-params(7)+params(7)*y(34));
  g1(22,30)=y(27)*(1-params(7)+params(7)*y(34));
  g1(22,34)=params(7)*y(30)*y(27);
  g1(22,37)=(-(y(23)*(1-params(5))));
  g1(23,16)=(-x(it_, 1));
  g1(23,28)=1;
  g1(23,46)=(-y(16));
  g1(24,20)=(-(y(35)*T227*params(15)*1/y(21)));
  g1(24,21)=(-(y(35)*T227*params(15)*(-y(20))/(y(21)*y(21))));
  g1(24,34)=1-x(it_, 2);
  g1(24,35)=(-T227);
  g1(24,47)=(-y(34));
  g1(25,16)=1;
  g1(25,17)=(-1);
  g1(25,18)=(-1);
  g1(25,20)=1;
  g1(25,28)=(-1);
  g1(26,32)=1;
  g1(26,34)=(-(1-x(it_, 1)));
  g1(26,46)=(-(1-y(34)));
  g1(27,17)=(-(((y(17)+y(28))*y(34)-(y(28)+y(17)*y(34)))/((y(17)+y(28))*(y(17)+y(28)))));
  g1(27,28)=(-((y(17)+y(28)-(y(28)+y(17)*y(34)))/((y(17)+y(28))*(y(17)+y(28)))));
  g1(27,33)=1;
  g1(27,34)=(-(y(17)/(y(17)+y(28))));
  g1(28,12)=1;
  g1(28,24)=(-1);
  g1(28,25)=(-1);
  g1(29,1)=1;
  g1(29,26)=(-1);
  g1(29,27)=(-1);
  g1(30,35)=1;
  g1(31,12)=(-((-(T249*getPowerDeriv(y(12),params(5),1)))/(T250*T250)));
  g1(31,1)=(-((-(y(21)*getPowerDeriv(y(1),1-params(5),1)))/(T248*T248)/T250));
  g1(31,21)=(-(1/T248/T250));
  g1(31,40)=1;
  g1(32,17)=(-(1/y(18)));
  g1(32,18)=(-((-(y(17)+y(28)))/(y(18)*y(18))));
  g1(32,28)=(-(1/y(18)));
  g1(32,41)=1;
end
if nargout >= 3,
  %
  % Hessian matrix
  %

  g2 = sparse([],[],[],32,2401);
end
if nargout >= 4,
  %
  % Third order derivatives
  %

  g3 = sparse([],[],[],32,117649);
end
end
