function [residual, g1, g2] = model_det1_static(y, x, params)
%
% Status : Computes static model for Dynare
%
% Inputs : 
%   y         [M_.endo_nbr by 1] double    vector of endogenous variables in declaration order
%   x         [M_.exo_nbr by 1] double     vector of exogenous variables in declaration order
%   params    [M_.param_nbr by 1] double   vector of parameter values in declaration order
%
% Outputs:
%   residual  [M_.endo_nbr by 1] double    vector of residuals of the static model equations 
%                                          in order of declaration of the equations
%   g1        [M_.endo_nbr by M_.endo_nbr] double    Jacobian matrix of the static model equations;
%                                                     columns: variables in declaration order
%                                                     rows: equations in order of declaration
%   g2        [M_.endo_nbr by (M_.endo_nbr)^2] double   Hessian matrix of the static model equations;
%                                                       columns: variables in declaration order
%                                                       rows: equations in order of declaration
%
%
% Warning : this file is generated automatically by Dynare
%           from model file (.mod)

residual = zeros( 32, 1);

%
% Model equations
%

T42 = y(6)/y(5)-(y(6))/(y(5));
T44 = params(11)/2*T42^2;
T93 = y(1)*params(4)/y(2);
T117 = (params(13)-1)/params(13);
T124 = x(4)*y(13)^T117+(1-x(4))*y(14)^T117;
T207 = exp(params(15)*(y(11)/y(12)-params(16)));
T228 = y(4)^(1-params(5));
T229 = y(12)/T228;
T230 = y(3)^params(5);
T282 = (-y(6))/(y(5)*y(5))-(-(y(6)))/((y(5))*(y(5)));
T285 = params(11)/2*T282*2*T42;
T298 = 1/y(5)-1/(y(5));
T300 = params(11)/2*2*T42*T298;
T351 = getPowerDeriv(T124,params(13)/(params(13)-1),1);
lhs =y(30);
rhs =log(y(1))-params(2)*y(3)^(1+params(3))/(1+params(3))+log(y(2))+y(30)*params(4);
residual(1)= lhs-rhs;
lhs =y(1)+y(7)+y(10)+y(22)*y(5)+y(6)+y(5)*T44;
rhs =y(3)*y(20)+y(29)*params(12)+params(14)*(x(2)*y(25)*y(11)+y(10)*x(3)*y(26));
residual(2)= lhs-rhs;
lhs =y(2);
rhs =y(29)*(1-params(12));
residual(3)= lhs-rhs;
lhs =y(29);
rhs =y(7)*y(23)+y(10)*y(26)*(1-x(3))+(y(22)*(1-params(1))+y(21))*(y(5)+y(6))+(x(2)*y(25)*y(11)+y(10)*x(3)*y(26))*(1-params(14));
residual(4)= lhs-rhs;
lhs =y(20);
rhs =y(1)*params(2)*y(3)^params(3);
residual(5)= lhs-rhs;
lhs =1;
rhs =y(23)*T93;
residual(6)= lhs-rhs;
residual(7) = min(y(23)-y(26)*(1-x(3)),y(10));
lhs =y(22)+T44-y(6)*params(11)*T42/y(5);
rhs =(y(22)*(1-params(1))+y(21))*T93;
residual(8)= lhs-rhs;
lhs =1+params(11)*T42;
rhs =(y(22)*(1-params(1))+y(21))*T93;
residual(9)= lhs-rhs;
lhs =y(4);
rhs =y(5)+y(6);
residual(10)= lhs-rhs;
lhs =y(5);
rhs =(1-params(1))*y(4);
residual(11)= lhs-rhs;
lhs =y(12);
rhs =T124^(params(13)/(params(13)-1));
residual(12)= lhs-rhs;
lhs =y(13)*y(27)^params(13);
rhs =y(12)*x(4)^params(13);
residual(13)= lhs-rhs;
lhs =y(14)*y(28)^params(13);
rhs =y(12)*(1-x(4))^params(13);
residual(14)= lhs-rhs;
lhs =y(13);
rhs =params(8)*y(17)^(1-params(5))*y(15)^params(5);
residual(15)= lhs-rhs;
lhs =y(8)+y(19);
rhs =params(6)*(y(20)*y(15)+y(21)*y(17));
residual(16)= lhs-rhs;
lhs =y(20)*y(15)*(1-params(6)+params(6)*y(24));
rhs =y(27)*y(13)*params(5);
residual(17)= lhs-rhs;
lhs =y(21)*y(17)*(1-params(6)+params(6)*y(24));
rhs =y(27)*y(13)*(1-params(5));
residual(18)= lhs-rhs;
lhs =y(14);
rhs =params(9)*y(18)^(1-params(5))*y(16)^params(5);
residual(19)= lhs-rhs;
lhs =y(9);
rhs =params(7)*(y(20)*y(16)+y(21)*y(18));
residual(20)= lhs-rhs;
lhs =y(20)*y(16)*(1-params(7)+y(25)*params(7));
rhs =y(28)*y(14)*params(5);
residual(21)= lhs-rhs;
lhs =y(21)*y(18)*(1-params(7)+y(25)*params(7));
rhs =y(28)*y(14)*(1-params(5));
residual(22)= lhs-rhs;
lhs =y(19);
rhs =y(7)*x(1);
residual(23)= lhs-rhs;
lhs =y(25)*(1-x(2));
rhs =y(26)*T207;
residual(24)= lhs-rhs;
lhs =y(7)+y(11);
rhs =y(8)+y(19)+y(9);
residual(25)= lhs-rhs;
lhs =y(23);
rhs =x(1)+y(25)*(1-x(1));
residual(26)= lhs-rhs;
lhs =y(24);
rhs =(y(19)+y(25)*y(8))/(y(8)+y(19));
residual(27)= lhs-rhs;
lhs =y(3);
rhs =y(15)+y(16);
residual(28)= lhs-rhs;
lhs =y(4);
rhs =y(17)+y(18);
residual(29)= lhs-rhs;
lhs =y(26);
rhs =params(10);
residual(30)= lhs-rhs;
lhs =y(31);
rhs =T229/T230;
residual(31)= lhs-rhs;
lhs =y(32);
rhs =(y(8)+y(19))/y(9);
residual(32)= lhs-rhs;
if ~isreal(residual)
  residual = real(residual)+imag(residual).^2;
end
if nargout >= 2,
  g1 = zeros(32, 32);

  %
  % Jacobian matrix
  %

  g1(1,1)=(-(1/y(1)));
  g1(1,2)=(-(1/y(2)));
  g1(1,3)=params(2)*getPowerDeriv(y(3),1+params(3),1)/(1+params(3));
  g1(1,30)=1-params(4);
  g1(2,1)=1;
  g1(2,3)=(-y(20));
  g1(2,5)=y(22)+T44+y(5)*T285;
  g1(2,6)=1+y(5)*T300;
  g1(2,7)=1;
  g1(2,10)=1-params(14)*x(3)*y(26);
  g1(2,11)=(-(params(14)*x(2)*y(25)));
  g1(2,20)=(-y(3));
  g1(2,22)=y(5);
  g1(2,25)=(-(params(14)*x(2)*y(11)));
  g1(2,26)=(-(params(14)*y(10)*x(3)));
  g1(2,29)=(-params(12));
  g1(3,2)=1;
  g1(3,29)=(-(1-params(12)));
  g1(4,5)=(-(y(22)*(1-params(1))+y(21)));
  g1(4,6)=(-(y(22)*(1-params(1))+y(21)));
  g1(4,7)=(-y(23));
  g1(4,10)=(-(y(26)*(1-x(3))+x(3)*y(26)*(1-params(14))));
  g1(4,11)=(-(x(2)*y(25)*(1-params(14))));
  g1(4,21)=(-(y(5)+y(6)));
  g1(4,22)=(-((1-params(1))*(y(5)+y(6))));
  g1(4,23)=(-y(7));
  g1(4,25)=(-((1-params(14))*x(2)*y(11)));
  g1(4,26)=(-(y(10)*(1-x(3))+(1-params(14))*y(10)*x(3)));
  g1(4,29)=1;
  g1(5,1)=(-(params(2)*y(3)^params(3)));
  g1(5,3)=(-(y(1)*params(2)*getPowerDeriv(y(3),params(3),1)));
  g1(5,20)=1;
  g1(6,1)=(-(y(23)*params(4)/y(2)));
  g1(6,2)=(-(y(23)*(-(y(1)*params(4)))/(y(2)*y(2))));
  g1(6,23)=(-T93);
  g1(7,10)=1-(y(10)>y(23)-y(26)*(1-x(3)));
  g1(7,23)=y(10)>y(23)-y(26)*(1-x(3));
  g1(7,26)=(y(10)>y(23)-y(26)*(1-x(3)))*(-(1-x(3)));
  g1(8,1)=(-((y(22)*(1-params(1))+y(21))*params(4)/y(2)));
  g1(8,2)=(-((y(22)*(1-params(1))+y(21))*(-(y(1)*params(4)))/(y(2)*y(2))));
  g1(8,5)=T285-(y(5)*y(6)*params(11)*T282-y(6)*params(11)*T42)/(y(5)*y(5));
  g1(8,6)=T300-(params(11)*T42+y(6)*params(11)*T298)/y(5);
  g1(8,21)=(-T93);
  g1(8,22)=1-(1-params(1))*T93;
  g1(9,1)=(-((y(22)*(1-params(1))+y(21))*params(4)/y(2)));
  g1(9,2)=(-((y(22)*(1-params(1))+y(21))*(-(y(1)*params(4)))/(y(2)*y(2))));
  g1(9,5)=params(11)*T282;
  g1(9,6)=params(11)*T298;
  g1(9,21)=(-T93);
  g1(9,22)=(-((1-params(1))*T93));
  g1(10,4)=1;
  g1(10,5)=(-1);
  g1(10,6)=(-1);
  g1(11,4)=(-(1-params(1)));
  g1(11,5)=1;
  g1(12,12)=1;
  g1(12,13)=(-(x(4)*getPowerDeriv(y(13),T117,1)*T351));
  g1(12,14)=(-(T351*(1-x(4))*getPowerDeriv(y(14),T117,1)));
  g1(13,12)=(-(x(4)^params(13)));
  g1(13,13)=y(27)^params(13);
  g1(13,27)=y(13)*getPowerDeriv(y(27),params(13),1);
  g1(14,12)=(-((1-x(4))^params(13)));
  g1(14,14)=y(28)^params(13);
  g1(14,28)=y(14)*getPowerDeriv(y(28),params(13),1);
  g1(15,13)=1;
  g1(15,15)=(-(params(8)*y(17)^(1-params(5))*getPowerDeriv(y(15),params(5),1)));
  g1(15,17)=(-(y(15)^params(5)*params(8)*getPowerDeriv(y(17),1-params(5),1)));
  g1(16,8)=1;
  g1(16,15)=(-(y(20)*params(6)));
  g1(16,17)=(-(y(21)*params(6)));
  g1(16,19)=1;
  g1(16,20)=(-(y(15)*params(6)));
  g1(16,21)=(-(y(17)*params(6)));
  g1(17,13)=(-(y(27)*params(5)));
  g1(17,15)=y(20)*(1-params(6)+params(6)*y(24));
  g1(17,20)=y(15)*(1-params(6)+params(6)*y(24));
  g1(17,24)=params(6)*y(20)*y(15);
  g1(17,27)=(-(y(13)*params(5)));
  g1(18,13)=(-(y(27)*(1-params(5))));
  g1(18,17)=y(21)*(1-params(6)+params(6)*y(24));
  g1(18,21)=y(17)*(1-params(6)+params(6)*y(24));
  g1(18,24)=params(6)*y(21)*y(17);
  g1(18,27)=(-(y(13)*(1-params(5))));
  g1(19,14)=1;
  g1(19,16)=(-(params(9)*y(18)^(1-params(5))*getPowerDeriv(y(16),params(5),1)));
  g1(19,18)=(-(y(16)^params(5)*params(9)*getPowerDeriv(y(18),1-params(5),1)));
  g1(20,9)=1;
  g1(20,16)=(-(y(20)*params(7)));
  g1(20,18)=(-(y(21)*params(7)));
  g1(20,20)=(-(y(16)*params(7)));
  g1(20,21)=(-(y(18)*params(7)));
  g1(21,14)=(-(y(28)*params(5)));
  g1(21,16)=y(20)*(1-params(7)+y(25)*params(7));
  g1(21,20)=y(16)*(1-params(7)+y(25)*params(7));
  g1(21,25)=params(7)*y(20)*y(16);
  g1(21,28)=(-(y(14)*params(5)));
  g1(22,14)=(-(y(28)*(1-params(5))));
  g1(22,18)=y(21)*(1-params(7)+y(25)*params(7));
  g1(22,21)=y(18)*(1-params(7)+y(25)*params(7));
  g1(22,25)=params(7)*y(21)*y(18);
  g1(22,28)=(-(y(14)*(1-params(5))));
  g1(23,7)=(-x(1));
  g1(23,19)=1;
  g1(24,11)=(-(y(26)*T207*params(15)*1/y(12)));
  g1(24,12)=(-(y(26)*T207*params(15)*(-y(11))/(y(12)*y(12))));
  g1(24,25)=1-x(2);
  g1(24,26)=(-T207);
  g1(25,7)=1;
  g1(25,8)=(-1);
  g1(25,9)=(-1);
  g1(25,11)=1;
  g1(25,19)=(-1);
  g1(26,23)=1;
  g1(26,25)=(-(1-x(1)));
  g1(27,8)=(-((y(25)*(y(8)+y(19))-(y(19)+y(25)*y(8)))/((y(8)+y(19))*(y(8)+y(19)))));
  g1(27,19)=(-((y(8)+y(19)-(y(19)+y(25)*y(8)))/((y(8)+y(19))*(y(8)+y(19)))));
  g1(27,24)=1;
  g1(27,25)=(-(y(8)/(y(8)+y(19))));
  g1(28,3)=1;
  g1(28,15)=(-1);
  g1(28,16)=(-1);
  g1(29,4)=1;
  g1(29,17)=(-1);
  g1(29,18)=(-1);
  g1(30,26)=1;
  g1(31,3)=(-((-(T229*getPowerDeriv(y(3),params(5),1)))/(T230*T230)));
  g1(31,4)=(-((-(y(12)*getPowerDeriv(y(4),1-params(5),1)))/(T228*T228)/T230));
  g1(31,12)=(-(1/T228/T230));
  g1(31,31)=1;
  g1(32,8)=(-(1/y(9)));
  g1(32,9)=(-((-(y(8)+y(19)))/(y(9)*y(9))));
  g1(32,19)=(-(1/y(9)));
  g1(32,32)=1;
  if ~isreal(g1)
    g1 = real(g1)+2*imag(g1);
  end
end
if nargout >= 3,
  %
  % Hessian matrix
  %

  g2 = sparse([],[],[],32,1024);
end
end
