// This is a small open economy real business cycle model.
// Allow both capital inflows and capital outflows

// Endogenous variables.

var cy co h k ko i d bs bp bfd bfl y ys yp hs hp ks kp bg
w rk qk  r rs rp rf ps pp wealth utility tfp bsshare;


// Shock innovations
varexo GAMMA TAUL TAUD PHI;

parameters DELTA PSIH ETA BETA ALPHA THETAS THETAP AS AP RSTAR OMEGAK TAU SIGMA THETAY PHIB BYBAR GAMMA0 GAMMA1 TAUL0 TAUL1 TAUD0 TAUD1 ALPHA_GAMMA ALPHA_TAUL ALPHA_TAUD PHI0 PHI1; 


DELTA = 1-0.9^10;
ETA = 2;
BETA = 0.96^10;% imply annualized domestic interest rate = 0.04
ALPHA = 0.5;
THETAS = 1;
THETAP = 1;
AS  = 1;
AP = 1.42;
RSTAR = 1.05^10;
SIGMA  = 3;
OMEGAK = 5;
PSIH = 24;
THETAY = 1;
TAU = 0.75;
PHIB  = 3;
BYBAR = 0.04;
GAMMA0 = 0.5;
TAUL0 =0.064712836159997;
TAUD0 =0.158448203605851;

GAMMA1 = GAMMA0;
TAUL1 =TAUL0;
TAUD1 =TAUD0;


ALPHA_GAMMA = 0;
ALPHA_TAUL  = 0;
ALPHA_TAUD  = 0;

PHI0 = 0.5;
PHI1 = 0.3;
/*
load respolicy
if usepolicy ==1
GAMMA0 = policy(1);
TAUL0 =   policy(2);
TAUD0 =   policy(3);
GAMMA1 =  policy(4);
TAUL1 =   policy(5);
TAUD1 =   policy(6);
ALPHA_GAMMA = policy(7);
ALPHA_TAUL  = policy(8);
ALPHA_TAUD  = policy(9);
PHI0 = policy(10);
PHI1 = policy(11);
end
*/
model(bytecode);
utility = log(cy)  - PSIH*h^(ETA+1)/(1+ETA) + log(co)  + BETA*utility(+1);

// Household sector
//y - c - i + (rf(-1) -1)*bf(-1) = bf - bf(-1);

cy + d + bfd + qk*ko + i + OMEGAK/2*(i/ko - steady_state(i)/steady_state(ko))^2*ko = w * h + wealth * TAU + THETAY*(TAUL(-1)*rp(-1)*bfl(-1) + TAUD(-1)*rf(-1)*bfd(-1));

co = wealth * (1-TAU);

wealth = r(-1)*d(-1) + (1-TAUD(-1))*rf(-1)*bfd(-1)  + (qk*(1-DELTA) + rk)*(ko(-1) + i(-1)) + (1-THETAY)*(TAUL(-1)*rp(-1)*bfl(-1) + TAUD(-1)*rf(-1)*bfd(-1));

w = PSIH*h^ETA *cy;

1 = BETA * cy/co(+1)*r;

//r = (1-TAUD)*rf;
min(r - (1-TAUD)*rf,bfd) = 0;

qk + OMEGAK/2*(i/ko - steady_state(i)/steady_state(ko))^2 - OMEGAK*(i/ko - steady_state(i)/steady_state(ko))*i/ko = BETA* cy/co(+1)*(qk(+1)*(1-DELTA)+rk(+1));

1 + OMEGAK*(i/ko - steady_state(i)/steady_state(ko)) = BETA* cy/co(+1)*(qk(+1)*(1-DELTA)+rk(+1));

k = ko + i;

ko = (1-DELTA)*k(-1);



// Consumption goods


y = (PHI*ys^((SIGMA - 1)/SIGMA) + (1-PHI)*yp^((SIGMA - 1)/SIGMA))^(SIGMA/(SIGMA - 1));

//y = ys^PHI*yp^(1-PHI);


ys*ps^SIGMA = PHI^SIGMA*y;

yp*pp^SIGMA = (1-PHI)^SIGMA*y;

//Intermediate goods

ys = AS*(ks)^(1-ALPHA)*(hs)^ALPHA;

bs + bg = THETAS*(w*hs + rk*ks);

w*hs*(1-THETAS+rs*THETAS) = ALPHA*ys*ps;

rk*ks*(1-THETAS+rs*THETAS) = (1-ALPHA)*ys*ps;

yp = AP*(kp)^(1-ALPHA)*(hp)^ALPHA;

bp = THETAP*(w*hp + rk*kp);

w*hp*(1-THETAP+rp*THETAP) = ALPHA*yp*pp;

rk*kp*(1-THETAP+rp*THETAP) = (1-ALPHA)*yp*pp;


// Banks

bg = min(d+bfl-bp,GAMMA*d);

//rp*(1-TAUL) = rf*exp(PHIB*(bfl/y-BYBAR));
min(rf*exp(PHIB*(bfl/y-BYBAR))-rp*(1-TAUL),bfl) = 0;

d + bfl = bg + bs + bp;

//r = 1*GAMMA+rp*(1-GAMMA);
r*d = 1*bg + rp*(d-bg);

rs = (1*bg + rp*bs)/(bg + bs);

h = hs + hp;

k(-1) = ks + kp;

rf = RSTAR;

tfp = y/(k(-1))^(1-ALPHA)/(h)^ALPHA;

bsshare = (bs+bg)/bp;
end;



steady_state_model;
[cy,co,h,k,ko,i,d,bs,bp,bfl,bfd,y,ys,yp,hs,hp,ks,kp,w,rk,qk,r,rs,rp,rf,ps,pp,wealth,bg]= solvess_det2(DELTA,PSIH,ETA,BETA,ALPHA,THETAS,THETAP,AS,AP,RSTAR,PHI,GAMMA,TAUL,TAUD,TAU,SIGMA,THETAY,PHIB,BYBAR);
utility = (log(cy)  - PSIH*h^(ETA+1)/(1+ETA) + log(co))/(1-BETA);
tfp = y/(k)^(1-ALPHA)/(h)^ALPHA;
bsshare = (bs+bg)/bp;
end;

initval;
GAMMA  = GAMMA0;
TAUL    = TAUL0;
TAUD    = TAUD0;
PHI     = PHI0;
end;
steady;
check;


endval;
GAMMA  = GAMMA1;
TAUL    = TAUL1;
TAUD    = TAUD1;
PHI     = PHI1;
end;
steady;
check;

V_1_100 = [1;2;3;4;5;6;7;8;9;10;11;12;13;14;15;16;17;18;19;20;21;22;23;24;25;26;27;28;29;30;31;32;33;34;35;36;37;38;39;40;41;42;43;44;45;46;47;48;49;50;51;52;53;54;55;56;57;58;59;60;61;62;63;64;65;66;67;68;69;70;71;72;73;74;75;76;77;78;79;80;81;82;83;84;85;86;87;88;89;90;91;92;93;94;95;96;97;98;99;100];

xxGAMMA = [GAMMA0+(GAMMA1-GAMMA0)*(1-ALPHA_GAMMA.^V_1_100)];
xxTAUD = [TAUD0+(TAUD1-TAUD0)*(1-ALPHA_TAUD.^V_1_100)];
xxTAUL   = [TAUL0+(TAUL1-TAUL0)*(1-ALPHA_TAUL.^V_1_100)];


shocks;
var GAMMA; 
periods 1:100;
values (xxGAMMA);
var TAUL; 
periods 1:100;
values (xxTAUL);
var TAUD; 
periods 1:100;
values (xxTAUD); 
end;



simul(periods = 200,maxit = 20,stack_solve_algo = 1);


figure;
subplot(2,3,1)
plot(bfd)
title('Capital outflows')
subplot(2,3,2)
plot(bfl)
title('Capital inflows')
subplot(2,3,3)
plot(tfp)
title('TFP')
subplot(2,3,4)
plot(y)
title('Output')
subplot(2,3,5)
plot(cy)
title('Young Consumption')
subplot(2,3,6)
plot(co)
title('Old Consumption')
figure;
plot(bs)
title('SOE market loans')
[oo_.endo_simul(30,1:2) oo_.endo_simul(30,201)]

