%% This file is used to display optimal liberalization policy following a decline in SOE share
dynare model_det1

clear
PHIchange = 1;
save res_tran1 PHIchange
ifopt = 0;
% set ifopt = 0 if you want to just use the saved optimal liberalization policy (res_tran1_PHIchange.mat); 
% set ifopt = 1 if you want to do the optimization yourself. 

%% solve for optimal liberalization policy
if ifopt == 1
%% calibration
GAMMA0 = 0.5;
TAUL0 =0.064712836159997;
TAUD0 =0.158448203605851;
GAMMA1 = GAMMA0;
TAUL1 =TAUL0;
TAUD1 =TAUD0;
ALPHA_GAMMA = 0;
ALPHA_TAUL  = 0;
ALPHA_TAUD  = 0;
 
%% optimal policy with liberalization of both capital outflows and financial repression
[x,FVAL,flag] = fminsearch(@(x) -welfare_tran1(GAMMA0, 0, TAUL0, TAUL0, TAUD0, 1-exp(x(1)), 0, ALPHA_TAUL, (atan(x(2))+pi/2)/pi,PHIchange),[log(1-0) tan(pi*0.5-pi/2)],optimset('MaxFunEvals',8000,'TolFun',1e-8));
opt_tdg0 = [0 1-exp(x(1)) 0  (atan(x(2))+pi/2)/pi -FVAL flag]
load res_tran1
save res_tran1 PHIchange  opt_tdg0
[x,FVAL,flag] = fminsearch(@(x) -welfare_tran1(GAMMA0, exp(x(1)), TAUL0, TAUL0, TAUD0, 1-exp(x(2)), (atan(x(3))+pi/2)/pi, ALPHA_TAUL, (atan(x(4))+pi/2)/pi,PHIchange),[log(1e-6) x(1) -30 x(2)],optimset('MaxFunEvals',8000,'TolFun',1e-8));
opt_tdg1 = [exp(x(1)) 1-exp(x(2)) (atan(x(3))+pi/2)/pi  (atan(x(4))+pi/2)/pi -FVAL flag]
load res_tran1
save res_tran1 PHIchange  opt_tdg0  opt_tdg1

%% optimal policy with liberalization of  both capital inflows and financial repression
[x,FVAL,flag] = fminsearch(@(x) -welfare_tran1(GAMMA0, 0, TAUL0, 1-exp(x(1)), TAUD0, TAUD0, 0, (atan(x(2))+pi/2)/pi, ALPHA_TAUD,PHIchange),[log(1+0.05) -30],optimset('MaxFunEvals',8000,'TolFun',1e-8));
opt_tlg0 = [0 1-exp(x(1)) 0  (atan(x(2))+pi/2)/pi -FVAL flag]
load res_tran1
save res_tran1 PHIchange  opt_tdg0 opt_tdg1 opt_tlg0

[x,FVAL,flag] = fminsearch(@(x) -welfare_tran1(GAMMA0, exp(x(1)), TAUL0, 1-exp(x(2)), TAUD0, TAUD0, (atan(x(3))+pi/2)/pi, (atan(x(4))+pi/2)/pi, ALPHA_TAUD,PHIchange),[log(1e-6) x(1) -30 x(2)],optimset('MaxFunEvals',8000,'TolFun',1e-8));
opt_tlg1 = [exp(x(1)) 1-exp(x(2)) (atan(x(3))+pi/2)/pi  (atan(x(4))+pi/2)/pi -FVAL flag]
load res_tran1
save res_tran1 PHIchange  opt_tdg0 opt_tdg1 opt_tlg0 opt_tlg1

%% optimal policy with full liberalization
[x,FVAL,flag] = fminsearch(@(x) -welfare_tran1(GAMMA0, 0, TAUL0, 1-exp(x(1)), TAUD0, 1-exp(x(2)), 0, (atan(x(3))+pi/2)/pi, (atan(x(4))+pi/2)/pi,PHIchange),[log(1-0) log(1-0)  tan(pi*0.5-pi/2) tan(pi*0.5-pi/2)],optimset('MaxIter', 10000,'MaxFunEvals',80000,'TolFun',1e-8));
vTAUL1 = 1-exp(x(1));
vTAUD1 = 1-exp(x(2));
vALPHA_TAUL = (atan(x(3))+pi/2)/pi;
vALPHA_TAUD = (atan(x(4))+pi/2)/pi;
welfare_diff = -FVAL;
opt_diff0=[0 vTAUL1 vTAUD1 0 vALPHA_TAUL vALPHA_TAUD welfare_diff flag]
load res_tran1
save res_tran1 PHIchange  opt_tdg0 opt_tdg1 opt_tlg0 opt_tlg1 opt_diff0

[x,FVAL,flag] = fminsearch(@(x) -welfare_tran1(GAMMA0, exp(x(1)), TAUL0, 1-exp(x(2)), TAUD0, 1-exp(x(3)), (atan(x(4))+pi/2)/pi, (atan(x(5))+pi/2)/pi, (atan(x(6))+pi/2)/pi,PHIchange),[log(1e-6) x(1) x(2)  -30 x(3) x(4)],optimset('MaxIter', 10000,'MaxFunEvals',80000,'TolFun',1e-8));
vGAMMA1 = exp(x(1));
vTAUL1 = 1-exp(x(2));
vTAUD1 = 1-exp(x(3));
vALPHA_GAMMA = (atan(x(4))+pi/2)/pi;
vALPHA_TAUL = (atan(x(5))+pi/2)/pi;
vALPHA_TAUD = (atan(x(6))+pi/2)/pi;
welfare_diff = -FVAL;
opt_diff1=[vGAMMA1 vTAUL1 vTAUD1 vALPHA_GAMMA vALPHA_TAUL vALPHA_TAUD welfare_diff flag]
load res_tran1
save res_tran1 PHIchange opt_tdg0 opt_tdg1 opt_tlg0 opt_tlg1 opt_diff0 opt_diff1

%% save optimal optimal liberalization policy
save res_tran1_PHIchange PHIchange PHIchange opt_tdg0 opt_tdg1 opt_tlg0 opt_tlg1 opt_diff0 opt_diff1

end

%% display results

dynare model_det2
welfare0 = oo_.endo_simul(30,2);%corrected welfare in case 0, where the directed lending constraint Bg<=Bs is binding.

load res_tran1_PHIchange
GAMMA0 = 0.5;
TAUL0 =0.064712836159997;
TAUD0 =0.158448203605851;

BETA = 0.96^10;
case0 = [ TAUD0 NaN TAUL0 NaN GAMMA0 NaN welfare0];
case2 = [TAUD0 NaN opt_tlg1(2) 1-opt_tlg1(4) opt_tlg1(1) 1-opt_tlg1(3) opt_tlg1(5)];
case4 = [opt_tdg1(2) 1-opt_tdg1(4) TAUL0 NaN  opt_tdg1(1) 1-opt_tdg1(3) opt_tdg1(5)];
case5 = [opt_diff1(3) 1-opt_diff1(6) opt_diff1(2) 1-opt_diff1(5) opt_diff1(1) 1-opt_diff1(4) opt_diff1(7)];
Res =  [case0;case2;case4;case5]';
Res(7,:) = exp((Res(7,:)-Res(7,1))*(1-BETA))-1;
Res

